#include <stdio.h>
#include <stdlib.h>

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include "spi.h"
#include "ST7735R_TFT.h"

void IO_init()
{
    // Init buttons and pull-up

    // Right
    DDRD  &= ~(1 << PD0);
    PORTD |= (1 << PD0);

    // Down
    DDRD  &= ~(1 << PD1);
    PORTD |= (1 << PD1);

    // Up
    DDRD  &= ~(1 << PD4);
    PORTD |= (1 << PD4);

    // Left
    DDRD  &= ~(1 << PD5);
    PORTD |= (1 << PD5);
}

int main(void)
{
    IO_init();

    // Init SPI for LCD.
    SPI_init();

    // Init LCD
    ST7735R_Begin();

    //Black Background
    ST7735R_FillRect(0, 0, ST7735R_WIDTH, ST7735R_HEIGHT, 0, 0, 0);

    // Init seed
    srand(0);

    int nr_width = 30;
    int line_height = 30;
    int x_start = 10;
    int y_start = 17;

    // game matrix 4x4
    int mat[4][4] = { {0}, {0}, {0}, {0} };

    // starting numbers
    mat[1][2] = 2048;
    mat[3][0] = 2048;
    mat[2][3] = 1024;

    // possible values
    int numbers[] = {0, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048};

    // background color for each value
    int color_r[] = {50, 128, 0, 128, 0};
    int color_g[] = {50, 64, 0, 0, 128};
    int color_b[] = {50, 0, 128, 0, 0};

    while (1) {

    	// Print game title
    	char mesaj[100];
        sprintf(mesaj, "REVERSE 2048");
        ST7735R_DrawText(15, 140, mesaj, 255, 255, 255, 0, 0, 0);

        // Print my name xD
        char nume[100];
        sprintf(nume, "VALENTIN CRISMARU");
        ST7735R_DrawText(15, 150, nume, 255, 255, 255, 0, 0, 0);

        // Print numbers and squares
        for (int y = y_start; y < 108; y += line_height) {
            for (int x = x_start; x < 101; x += nr_width) {

                int nr = mat[y/line_height][x/nr_width], i;

                for (i = 0; i < 12; i++) {

                    if (numbers[i] == nr){

                        break;
                    }
                }

                // Square Background
                ST7735R_FillRect(x - 3, y - 10, x + 23 , y + 13, color_r[i], color_g[i], color_b[i]);

                // Number
                if (nr) {

                    char buf[10];

                    if (nr < 100) {

                        sprintf(buf, " %d", nr);
                    }
                    else {

                        sprintf(buf, "%d", nr);
                    }

                    ST7735R_DrawText(x, y, buf, 255, 255, 255, color_r[i], color_g[i], color_b[i]);
                }
            }
        }

        // Wait for button to be pushed
        while (1) {

            int pushed = 0, change = 0;

            //right
            if ((PIND & (1 << PD0)) == 0) {

                for (int i = 0; i < 4; i++) {
                    for (int j = 2; j >= 0; j--) {
                        for (int k = 0; k < 3 - j; k++) {

                            if (mat[i][j + k] && mat[i][j + k] == mat[i][j + 1 + k]) {

                                mat[i][j + k] = 0;
                                mat[i][j + 1 + k] = mat[i][j + 1 + k] / 2;
                                change = 1;

                            } else if (mat[i][j + k] && !mat[i][j + 1 + k]) {

                                mat[i][j + 1 + k] = mat[i][j + k];
                                mat[i][j + k] = 0;
                                change = 1;
                            }
                        }
                    }
                }
                _delay_ms(100);
                pushed = 1;
            }

            //down
            if ((PIND & (1 << PD1)) == 0) {

                for (int i = 2; i >= 0; i--) {
                    for (int j = 0; j < 4; j++) {
                        for (int k = 0; k < 3 - i; k++) {

                            if (mat[i + k][j] && mat[i + k][j] == mat[i + 1 + k][j]) {

                                mat[i + k][j] = 0;
                                mat[i + 1 + k][j] = mat[i + 1 + k][j] / 2;
                                change = 1;

                            } else if (mat[i + k][j] && !mat[i + 1 + k][j]) {

                                mat[i + 1 + k][j] = mat[i + k][j];
                                mat[i + k][j] = 0;
                                change = 1;
                            }
                        }
                    }
                }
                _delay_ms(100);
                pushed = 1;
            }

            //up
            if ((PIND & (1 << PD4)) == 0) {

                for (int i = 1; i < 4; i++) {
                    for (int j = 0; j < 4; j++) {
                        for (int k = 0; k < i; k++) {

                            if (mat[i - k][j] && mat[i - k][j] == mat[i - 1 - k][j]) {

                                mat[i - k][j] = 0;
                                mat[i - 1 - k][j] = mat[i - 1 - k][j] / 2;
                                change = 1;

                            } else if (mat[i - k][j] && !mat[i - 1 - k][j]) {

                                mat[i - 1 - k][j] = mat[i - k][j];
                                mat[i - k][j] = 0;
                                change = 1;
                            }
                        }
                    }
                }
                _delay_ms(100);
                pushed = 1;
            }

            //left
            if ((PIND & (1 << PD5)) == 0) {

                for (int i = 0; i < 4; i++) {
                    for (int j = 1; j < 4; j++) {
                        for (int k = 0; k < j; k++) {

                            if (mat[i][j - k] && mat[i][j - k] == mat[i][j - 1 - k]) {

                                mat[i][j - k] = 0;
                                mat[i][j - 1 - k] = mat[i][j - 1 - k] / 2;
                                change = 1;

                            } else if (mat[i][j - k] && !mat[i][j - 1 - k]) {

                                mat[i][j - 1 - k] = mat[i][j - k];
                                mat[i][j - k] = 0;
                                change = 1;
                            }
                        }
                    }
                }

                _delay_ms(100);
                pushed = 1;
            }

            // check if player won

            int won = 0;

            for (int i = 0; i < 4; i++) {
            	for (int j = 0; j < 4; j++) {

                    if (mat[i][j] == 1) {

                        won = 1;
                    }
                }
            }

            if (won) {

                char won_message[100];
                sprintf(won_message, "YOU WON!");
                ST7735R_DrawText(15, 130, won_message, 255, 255, 255, 0, 0, 0);

                while(1);
            }


            // check if game is over

            int over = 1;

            for (int i = 0; i < 4; i++) {
                for (int j = 0; j < 4; j++) {

                    if (!mat[i][j]) {

                        over = 0;
                    }
                }
            }

            for (int i = 1; i < 3; i++) {
                for (int j = 1; j < 3; j++) {

                    int aux = mat[i][j];

                    if (aux==mat[i+1][j] ||
                        aux==mat[i-1][j] ||
                        aux==mat[i][j+1] ||
                        aux==mat[i][j-1]) {

                        over = 0;
                    }
                }
            }

            if (mat[0][0]==mat[0][1] ||
                mat[0][0]==mat[1][0]) {

                over = 0;
            }
            if (mat[3][3]==mat[3][2] ||
                mat[3][3]==mat[2][3]) {

                over = 0;
            }
            if (mat[0][3]==mat[0][2] ||
                mat[0][3]==mat[1][3]) {

                over = 0;
            }
            if (mat[3][0]==mat[2][0] ||
                mat[3][0]==mat[3][1]){

                over = 0;
            }

            if (mat[1][0]==mat[2][0] ||
                mat[1][3]==mat[2][3]) {

                over = 0;
            }
            if (mat[0][1]==mat[0][2] ||
                mat[3][1]==mat[3][2]) {

                over = 0;
            }

            if (over) {

                char over_message[100];
                sprintf(over_message, "YOU FAILED !");
                ST7735R_DrawText(15, 130, over_message, 255, 255, 255, 0, 0, 0);

                while(1);
            }


            // print new tile
            if (pushed && change) {

                int x, y;

                // get random new empty position

                while (1) {

                    x = rand() % 4;
                    y = rand() % 4;

                    if (!mat[x][y]){

                        break;
                    }
                }

                mat[x][y] = 512 * (1 + rand() % 2);

                break;
            }
        }
    }
    return 0;
}
